/*
 ppc1.c - simple program to control the parallelport
 	  
 compile with: gcc -O2 -lm ppc1.c -o ppc1

 Copyright (c) 1999 by Carsten Voss <cavofl@t-online.de>. 

* This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

* This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

* You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to the
  Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <asm/io.h>
#include <math.h>

#define BASE	0x378	/* baseregister of parport */
#define DATA	BASE+0	/* dataregister */
#define STATUS	BASE+1	/* statusregister */
#define CONTROL	BASE+2	/* controlregister */

/* function definition */
char* dec2bin(int dec);
void prbin(char *text, int dec);

int main(int argc, char *argv[])
{
  int c, bit, mode = 0, setdata = 0, data = 0;
  int reg, oldreg, value, resval, debug = 0;
  
  /* parse options */
  for (;;) {
	if (-1 == (c = getopt(argc, argv, "vhs:c:d:")))
	    break;
	switch (c) {
	case 's' :
		mode = 1;
		bit = atoi(optarg);
		if (bit < 0 || bit > 7) {
		  fprintf(stderr, "Invalid value for s: Should be 0-7.\n");
		  exit(-1);
		}
		break;
	case 'c' :
		mode = 2;
		bit = atoi(optarg);
		if (bit < 0 || bit > 7) {
		  fprintf(stderr, "Invalid value for c: Should be 0-7.\n");
		  exit(-1);
		}
		break;
	case 'd' :
		setdata = 1;
		data = atoi(optarg);
		if (data < 0 || data > 255) {
		  fprintf(stderr, "Invalid value for d: Should be 0-255.\n");
		  exit (-1);
		}
		break;
	case 'v' :
		debug = 1;
		break;
	case 'h' :
		fprintf(stderr,
		"\nUsage: %s [-vh] [-s bit] [-c bit] [-d data]\n"
		"\n If no arguments are given, the value of the statusregister"
		"\n will be written to stdio.\n"
		"\n -v         verbose\n"
		"\n -h         show this text\n"
		"\n -s n       set bit n of controlregister (n = 0-7)\n"
		"\n -c n       clear bit n of controlregister (n = 0-7)\n"
		"\n -d n       write n to dataport an pull RESET to low for"
		"\n            some microseconds (n = 0-255)\n\n", argv[0]);
		exit(-1);
		break;
	default :
		exit(-1);		
	}
  }
    
  /* get permission for the port */
  if (ioperm(BASE, 3, 1)) {
	fprintf(stderr, "Can't get io-permissions for port: 0x%x\n" \
			"Run as root?\n", BASE);
	exit(-1);
  }
  
  switch (mode) {
  case 0 :
  	reg = inb(STATUS);
	prbin("S", reg);
	break;
  case 1 :
  	oldreg = inb(CONTROL);
	if (debug)
	  prbin("SetCTRL before: ", oldreg);
	value = ((int)pow(2, bit) | oldreg);
	outb(value, CONTROL);
	if (debug)
	  prbin("SetCTRL after : ", value);
	break;
  case 2 :
  	oldreg = inb(CONTROL);
	if (debug)
	  prbin("ClearCTRL before: ", oldreg);
	value = (~(int)pow(2, bit) & oldreg);
	outb(value, CONTROL);
	if (debug)
	  prbin("ClearCTRL after : ", value);
	break;
  }
    
  if (setdata == 1) {
    oldreg = inb(CONTROL);
        
    /* set ExtModeDir 0 */
    value = (oldreg & ~32);
    outb(value, CONTROL);
        
    /* write data */
    outb(data, DATA);
        
    /* set RESET to 0, to latch the data into the FlipFlop */
    resval = (value & ~4);
    outb(resval, CONTROL);
    usleep(100);
    resval = (value | 4);
    outb(resval, CONTROL);
        
    /* restore CTRL-Register */
    outb(oldreg, CONTROL);
    }
    
  return (0);
}

/*
** textoutput and call to dec2bin
*/
void prbin(char *text, int dec)
{
	char *out;
	out = dec2bin(dec);
	printf("%s:%d:%s\n", text, dec, out);
	free(out); 
}

/* 
** simple decimal to binary converter
*/
char* dec2bin(int dec)
{
	int x;
	int bit = 128;
	char* bin = (char*)calloc(9, sizeof(char));

	for (x = 0; x < 8; x++)
		{
		 	if (dec & bit)
		 		bin[x] = '1';
		 	else 
				bin[x] = '0';
 
			bit = bit >> 1;
		 }
	bin[x] = '\0'; /* terminate the string */
	return bin;
}
