#!/usr/bin/tclsh
#
###############################################################################
#
# lcdt1.tcl
#
# Controlprogram for the LCD-Panel, the Keys and the LEDs.
# 
# Copyright (c) 1999 by Carsten Voss <cavofl@t-online.de>.
#
# * This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
# * This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
# * You should have received a copy of the GNU General Public License
#   along with this program; see the file COPYING.  If not, write to the
#   Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
###############################################################################
#
# WARNING: THIS IS REALLY SLUGGISH BETA-SOFTWARE! USE AT YOUR OWN RISC!
#
###############################################################################

# path to mailbox
set mbox {/var/spool/mail/vossy}

# path to voicebox
set vbox {/var/spool/vbox/vossy/incoming}

# the ppc program
set ppc {/usr/local/sbin/ppc1}

# the playcommand for the vbox-files
set player {auplay}

# the logfile for last call
set lastcall {/var/log/lastcall}

# delay
set delay 100

# menudelay
set mdelay 500

# updatedelay
set udelay 500

###########################################################################

set ledstat 0

# the menus
#
#             1234567890123456
set mainmenu { AB  ML  IN  LI}
set vboxmenu { <-  ->  |>   }
set mailmenu { <-  ->       }
set infomenu { LO  UP  LC   }

#
# writes text to display
#
proc writedisp { text } {
	global lcd
	puts $lcd $text
	flush $lcd
}

proc cleardisp {} {
	global lcd
	puts -nonewline $lcd "c"
	flush $lcd
}

proc homedisp {} {
	global lcd
	puts -nonewline $lcd "h"
	flush $lcd
}

proc resetdisp {} {
	global lcd
	puts -nonewline $lcd "R"
	flush $lcd
	after 500
}

#
# turns backlight on or off (on=1, off=0)
#
proc backlight { onoff } {
	global ppc light
	if {$onoff == 0} {
	  set args "-c 3"
	  set light 0
	}
	if {$onoff == 1} {
	  set args "-s 3"
	  set light 1
	}
	if [catch {exec $ppc $args} rc] {
	  puts stderr "Error calling $ppc: $rc"
	}
}

#
# setleds (led=0-3 onoff=0,1(on))
#
proc setleds { led onoff } {
	global ppc ledstat
	if {$onoff == 1} {
	  set tmpdata [format "%1.0f" [expr pow(2, $led)]]
	  set data [expr $tmpdata | $ledstat]
	} else {
	  set tmpdata [format "%1.0f" [expr pow(2, $led)]]
	  set data [expr ~$tmpdata & $ledstat]
	}
	set ledstat $data  
	#puts stdout "LEDDATA: $data $led"
	if [catch {exec $ppc -d $data} rc] {
	  puts stderr "Error calling $ppc: $rc"
	}
}

#
# inital stuff
#
proc init {} {
	global lcd
	backlight 1
	resetdisp
	setleds 0 0
	setleds 1 0
	setleds 2 0
	setleds 3 0
	writedisp "   Welcome to\n lcdt1.tcl v0.1"
	after 2000
	cleardisp
	writedisp " Written 4/1999 by  Carsten Voss"
	after 2000
	cleardisp
	#backlight 0
}

#
# vbox control menu
#
proc dovbox {} {
  global ppc delay mdelay vbox vboxmenu player
  cleardisp
  setleds 0 0
  setleds 1 0
  after $mdelay
  
  set count 1
  if [catch {glob $vbox/*} vblist] {
    return 1
  } else {
  foreach vbname $vblist {
    set vfile($count) [lindex $vblist [expr $count -1]]
    set vfentry($count) [lindex [split $vfile($count) -] 1]
    #puts stdout "$count: $vfile($count)"
    incr count
  }
  }
  incr count -1
  set act_cnt $count
  
  writedisp "$act_cnt:$vfentry($act_cnt)\n$vboxmenu"
  checkifnew $vfile($act_cnt)
  
  # menu loop:
  set halt 0
  while {!$halt} {
    if [catch {exec $ppc} rc] {
      puts stderr "Error calling $ppc: $rc"
      exit -1
    }
    set bit [expr ~[lindex [split $rc :] 1] & 120]
    switch -exact -- $bit {
	 8 { if {$act_cnt <= 1} {
	 	writedisp "Can't do that.   \n"
		after 1500
		cleardisp
		writedisp "$act_cnt:$vfentry($act_cnt)\n$vboxmenu"
		checkifnew $vfile($act_cnt)
	     } else {
	        incr act_cnt -1
		after $mdelay
		cleardisp
		writedisp "$act_cnt:$vfentry($act_cnt)\n$vboxmenu"
		checkifnew $vfile($act_cnt)
	     }	        
	   }
	16 { if {$act_cnt >= $count} {
	 	writedisp "Can't do that.   \n"
		after 1500
		cleardisp
		writedisp "$act_cnt:$vfentry($act_cnt)\n$vboxmenu"
		checkifnew $vfile($act_cnt)
	     } else {
	        incr act_cnt
		after $mdelay
		cleardisp
		writedisp "$act_cnt:$vfentry($act_cnt)\n$vboxmenu"
		checkifnew $vfile($act_cnt)
	     }
	   }
	32 { cleardisp
	     writedisp "  Now playing\n   message $act_cnt"
	     if [catch {exec $player $vfile($act_cnt) 2} rc] {
		puts stderr "Error calling $player: $rc"
	     }
	     if [catch {exec chmod +x $vfile($act_cnt)} rc] {
	        puts stderr "Error calling chmod: $rc"
	     }
	     checkifnew $vfile($act_cnt)  
	     after 500
	     cleardisp
	     writedisp "$act_cnt:$vfentry($act_cnt)\n$vboxmenu"
	   }
	64 { return }
    }
    after $delay
  }
}

#
# checks if vbox-message is new or not, and sets LED 2 if so.
#
proc checkifnew { message } {
  if {![file executable $message]} {
    setleds 2 1
  } else {
    setleds 2 0
  }
}

#
# mailbox control menu
#
proc domail {} {
  global ppc delay mdelay mbox mailmenu
  setleds 0 0
  setleds 1 0
  setleds 2 0
  cleardisp
  after $mdelay
  set mcnt 1
  set mtext(1) "No mail.\n"
  if [catch {open $mbox {RDONLY}} mfd] {
    set mtext(1) "No Spoolfile.\n"
  } else {
    foreach line [split [read $mfd] \n] {
      if {[regexp ^From: $line] == 1} {
        set mtext($mcnt) [string trim [lindex [split $line <] 1]]
	set mtext($mcnt) [string trimright $mtext($mcnt) >]
	set mtext($mcnt) [string range $mtext($mcnt) 0 13]
	incr mcnt
      } 
    }
  close $mfd
  }
  incr mcnt -1
  set acnt 1
    
  writedisp "$acnt:$mtext(1)$mailmenu"
  
  set halt 0
  while {!$halt} {
    if [catch {exec $ppc} rc] {
      puts stderr "Error calling $ppc: $rc"
      exit -1
    }
    set bit [expr ~[lindex [split $rc :] 1] & 120]
    switch -exact -- $bit {
	 8 { if {$acnt <= 1} {
	 	writedisp "Can't do that.  \n"
		after 1500
		cleardisp
		writedisp "$acnt:$mtext($acnt)$mailmenu"
	     } else {
	        incr acnt -1
	        after $mdelay
	        writedisp "$acnt:$mtext($acnt)$mailmenu"
	     }
	   }
	16 { if {$acnt >= $mcnt} {
		writedisp "Can't do that.  \n"
		after 1500
		cleardisp
		writedisp "$acnt:$mtext($acnt)$mailmenu"
	     } else {
	        incr acnt
	        after $mdelay
		cleardisp
	        writedisp "$acnt:$mtext($acnt)$mailmenu"
	     }
	   }
	32 { 
	   }
	64 { return }
    }
    after $delay
  }
  
}

#
# info control menu
#
proc doinfo {} {
  global infomenu delay mdelay ppc lastcall lcall
  setleds 0 0
  setleds 1 0
  setleds 2 0
  cleardisp
  after $mdelay
  writedisp "   Info-Menu:\n$infomenu"
  set halt 0
  while {!$halt} {
    if [catch {exec $ppc} rc] {
      puts stderr "Error calling $ppc: $rc"
      exit -1
    }
    set bit [expr ~[lindex [split $rc :] 1] & 120]
    switch -exact -- $bit {
	8 { if [catch {exec cat /proc/loadavg} rc] {
		puts stderr "Error reading /proc/loadavg: $rc"
	    }
	    set l1 [string trim [lindex [split $rc] 0]]
	    set l2 [string trim [lindex [split $rc] 1]]
	    set l3 [string trim [lindex [split $rc] 2]]
	    homedisp
	    writedisp "$l1  $l2  $l3"
	   }
	16 { if [catch {exec uptime} rc] {
		puts stderr "Error calling uptime: $rc"
	     }
	     set ut [string trim [lindex [split $rc ,] 1]]
	     set ud [string trim [lindex [split $rc ,] 0]]
	     set uu [string trim [lindex [split $ud m] 1]]
	     set up "$uu  $ut"
	     set up [format "%16s" $up]
	     homedisp
	     writedisp $up
	   }
	32 { set ncnum [format "%16s" [lindex [split $lcall] 3]]
	     homedisp
	     writedisp $ncnum	
	   }
	64 { return }
    }
    after $delay
  }
}

#
# toggles backlight on or off
#
proc dolight {} {
	global light
	if {$light == 0} {
	  backlight 1
	} else {
	  backlight 0
	}	
}	

#
# check mailbox
#
proc checkmail {} {
  global mbox
  set mcnt 0
  if [catch {open $mbox {RDONLY}} mfd] {
    set mcnt 0
  } else {
   foreach line [split [read $mfd] \n] {
     if {[regexp ^From: $line] == 1} {
       incr mcnt
     }
   }
  close $mfd
  }
  
  if {$mcnt > 0} {
    setleds 1 1
  } else {
    setleds 1 0
  }
  return $mcnt
}

#
# check vbox
#
proc checkvbox {} {
  global vbox
  set tot_cnt 0
  set new_cnt 0
  if [catch {glob $vbox/*} vblist ] {
    set tot_cnt 0
    set new_cnt 0
  } else {
  foreach vbname $vblist {
    if {![file executable $vbname]} {
      incr new_cnt
    }
    incr tot_cnt
  }
  }
  if {$new_cnt > 0} {
    setleds 0 1
  } else {
    setleds 0 0
  }
  set retval "$new_cnt/$tot_cnt"
  return $retval
}

#
# shows statusline in main menu
#
proc showstatus {} {
  global mainmenu mdelay
  cleardisp
  after $mdelay
  set mails [checkmail]
  set vbstat [checkvbox]
  set ncstat [checkcalls]
  writedisp "A:$vbstat M:$mails  $ncstat$mainmenu"
}

#
# checkcalls
#
proc checkcalls {} {
  global lastcall lcall
  set newcall 0
  if [catch {open $lastcall {RDONLY}} cfd] {
    set lcall "Error"
  } else {
    set lcall [read $cfd]
    close $cfd
  }
  set ntime [lindex [split $lcall] 1]
  return $ntime
}

# open lcd
if [catch {open "/dev/lcd" {WRONLY NONBLOCK}} lcd] {
	puts stderr "Can't open /dev/lcd: $lcd"
	exit -1
}

set update 0

init
#backlight 1
#cleardisp
showstatus

# main loop:
set halt 0
while {!$halt} {
  if [catch {exec $ppc} rc] {
    puts stderr "Error calling $ppc: $rc"
    exit -1
  }
  set bit [expr ~[lindex [split $rc :] 1] & 120]
  switch -exact -- $bit {
	8 { dovbox 
	    showstatus
	  }
	16 { domail
	     showstatus  
	   }
	32 { doinfo
	     showstatus
	   }
	64 { dolight; after $mdelay }
  }
  
after $delay
incr update
if {$update >= $udelay} {
  set update 0
  showstatus
}
}

close $lcd
